@@ -31,16 +31,15 @@ import txaio
 import inspect
 
 from autobahn import wamp
-from autobahn.util import IdGenerator, ObservableMixin
+from autobahn.util import public, IdGenerator, ObservableMixin
 from autobahn.wamp import uri
 from autobahn.wamp import message
 from autobahn.wamp import types
 from autobahn.wamp import role
 from autobahn.wamp import exception
 from autobahn.wamp.exception import ApplicationError, ProtocolError, SessionNotReady, SerializationError
-from autobahn.wamp.interfaces import IApplicationSession  # noqa
-from autobahn.wamp.types import SessionDetails
-from autobahn.wamp.cryptobox import EncryptedPayload
+from autobahn.wamp.interfaces import ISession, IPayloadCodec  # noqa
+from autobahn.wamp.types import SessionDetails, CloseDetails, EncodedPayload
 from autobahn.wamp.request import \
     Publication, \
     Subscription, \
@@ -100,8 +99,8 @@ class BaseSession(ObservableMixin):
         self._authmethod = None
         self._authprovider = None
 
-        # end-to-end encryption keyring
-        self._keyring = None
+        # payload transparency codec
+        self._payload_codec = None
 
         # generator for WAMP request IDs
         self._request_id_gen = IdGenerator()
@@ -125,15 +124,16 @@ class BaseSession(ObservableMixin):
 
         :param request_type: The request type this WAMP error message is for.
         :type request_type: int
+
         :param request: The request ID this WAMP error message is for.
         :type request: int
+
         :param exc: The exception.
         :type exc: Instance of :class:`Exception` or subclass thereof.
+
         :param tb: Optional traceback. If present, it'll be included with the WAMP error message.
         :type tb: list or None
         """
-        assert(enc_algo is None or enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX)
-
         args = None
         if hasattr(exc, 'args'):
             args = list(exc.args)  # make sure tuples are made into lists
@@ -156,18 +156,18 @@ class BaseSession(ObservableMixin):
             else:
                 error = u"wamp.error.runtime_error"
 
-        encrypted_payload = None
-        if self._keyring:
-            encrypted_payload = self._keyring.encrypt(False, error, args, kwargs)
+        encoded_payload = None
+        if self._payload_codec:
+            encoded_payload = self._payload_codec.encode(False, error, args, kwargs)
 
-        if encrypted_payload:
+        if encoded_payload:
             msg = message.Error(request_type,
                                 request,
                                 error,
-                                payload=encrypted_payload.payload,
-                                enc_algo=encrypted_payload.algo,
-                                enc_key=encrypted_payload.pkey,
-                                enc_serializer=encrypted_payload.serializer)
+                                payload=encoded_payload.payload,
+                                enc_algo=encoded_payload.enc_algo,
+                                enc_key=encoded_payload.enc_key,
+                                enc_serializer=encoded_payload.enc_serializer)
         else:
             msg = message.Error(request_type,
                                 request,
@@ -192,16 +192,16 @@ class BaseSession(ObservableMixin):
         exc = None
         enc_err = None
 
-        if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
+        if msg.enc_algo:
 
-            if not self._keyring:
-                log_msg = u"received encrypted payload, but no keyring active"
+            if not self._payload_codec:
+                log_msg = u"received encoded payload, but no payload codec active"
                 self.log.warn(log_msg)
-                enc_err = ApplicationError(ApplicationError.ENC_NO_KEYRING_ACTIVE, log_msg, enc_algo=msg.enc_algo)
+                enc_err = ApplicationError(ApplicationError.ENC_NO_PAYLOAD_CODEC, log_msg, enc_algo=msg.enc_algo)
             else:
                 try:
-                    encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
-                    decrypted_error, msg.args, msg.kwargs = self._keyring.decrypt(True, msg.error, encrypted_payload)
+                    encoded_payload = EncodedPayload(msg.payload, msg.enc_algo, msg.enc_serializer, msg.enc_key)
+                    decrypted_error, msg.args, msg.kwargs = self._payload_codec.decode(True, msg.error, encoded_payload)
                 except Exception as e:
                     self.log.warn("failed to decrypt application payload 1: {err}", err=e)
                     enc_err = ApplicationError(
@@ -269,16 +269,15 @@ class BaseSession(ObservableMixin):
         return exc
 
 
+@public
 class ApplicationSession(BaseSession):
     """
     WAMP endpoint session.
     """
 
-    log = txaio.make_logger()
-
     def __init__(self, config=None):
         """
-        Constructor.
+        Implements :func:`autobahn.wamp.interfaces.ISession`
         """
         BaseSession.__init__(self)
         self.config = config or types.ComponentConfig(realm=u"default")
@@ -307,11 +306,22 @@ class ApplicationSession(BaseSession):
         # incoming invocations
         self._invocations = {}
 
-    def set_keyring(self, keyring):
+    @public
+    def set_payload_codec(self, payload_codec):
         """
+        Implements :func:`autobahn.wamp.interfaces.ISession.set_payload_codec`
         """
-        self._keyring = keyring
+        assert(payload_codec is None or isinstance(payload_codec, IPayloadCodec))
+        self._payload_codec = payload_codec
 
+    @public
+    def get_payload_codec(self):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ISession.get_payload_codec`
+        """
+        return self._payload_codec
+
+    @public
     def onOpen(self, transport):
         """
         Implements :func:`autobahn.wamp.interfaces.ITransportHandler.onOpen`
@@ -328,23 +338,34 @@ class ApplicationSession(BaseSession):
             None,
         )
 
+    @public
     def onConnect(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.onConnect`
         """
         self.join(self.config.realm)
 
-    def join(self, realm, authmethods=None, authid=None, authrole=None, authextra=None):
+    @public
+    def join(self,
+             realm,
+             authmethods=None,
+             authid=None,
+             authrole=None,
+             authextra=None,
+             resumable=None,
+             resume_session=None,
+             resume_token=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.join`
         """
-        # FIXME
-        if six.PY2 and type(realm) == str:
-            realm = six.u(realm)
-        if six.PY2 and type(authid) == str:
-            authid = six.u(authid)
-        if six.PY2 and type(authrole) == str:
-            authrole = six.u(authrole)
+        assert(realm is None or type(realm) == six.text_type)
+        assert(authmethods is None or type(authmethods) == list)
+        if type(authmethods) == list:
+            for authmethod in authmethods:
+                assert(type(authmethod) == six.text_type)
+        assert(authid is None or type(authid) == six.text_type)
+        assert(authrole is None or type(authrole) == six.text_type)
+        assert(authextra is None or type(authextra) == dict)
 
         if self._session_id:
             raise Exception("already joined")
@@ -357,9 +378,18 @@ class ApplicationSession(BaseSession):
         self._goodbye_sent = False
 
         # send HELLO message to router
-        msg = message.Hello(realm, role.DEFAULT_CLIENT_ROLES, authmethods, authid, authrole, authextra)
+        msg = message.Hello(realm=realm,
+                            roles=role.DEFAULT_CLIENT_ROLES,
+                            authmethods=authmethods,
+                            authid=authid,
+                            authrole=authrole,
+                            authextra=authextra,
+                            resumable=resumable,
+                            resume_session=resume_session,
+                            resume_token=resume_token)
         self._transport.send(msg)
 
+    @public
     def disconnect(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.disconnect`
@@ -367,38 +397,27 @@ class ApplicationSession(BaseSession):
         if self._transport:
             self._transport.close()
 
+    @public
     def is_connected(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.is_connected`
         """
         return self._transport is not None
 
+    @public
     def is_attached(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.is_attached`
         """
         return self._transport is not None and self._session_id is not None
 
+    @public
     def onUserError(self, fail, msg):
         """
-        This is called when we try to fire a callback, but get an
-        exception from user code -- for example, a registered publish
-        callback or a registered method. By default, this prints the
-        current stack-trace and then error-message to stdout.
-
-        ApplicationSession-derived objects may override this to
-        provide logging if they prefer. The Twisted implemention does
-        this. (See :class:`autobahn.twisted.wamp.ApplicationSession`)
-
-        :param fail: instance implementing txaio.IFailedFuture
-
-        :param msg: an informative message from the library. It is
-            suggested you log this immediately after the exception.
+        Implements :func:`autobahn.wamp.interfaces.ISession.onUserError`
         """
-        if isinstance(fail.value, exception.ApplicationError):
-            # silence on errors raised explicitly from the app
-            # previous code: self.log.error(fail.value.error_message())
-            pass
+        if False and isinstance(fail.value, exception.ApplicationError):
+            self.log.error(fail.value.error_message())
         else:
             self.log.error(
                 u'{msg}: {traceback}',
@@ -443,7 +462,16 @@ class ApplicationSession(BaseSession):
                 self._session_id = msg.session
                 self._router_roles = msg.roles
 
-                details = SessionDetails(self._realm, self._session_id, msg.authid, msg.authrole, msg.authmethod, msg.authprovider, msg.authextra)
+                details = SessionDetails(realm=self._realm,
+                                         session=self._session_id,
+                                         authid=msg.authid,
+                                         authrole=msg.authrole,
+                                         authmethod=msg.authmethod,
+                                         authprovider=msg.authprovider,
+                                         authextra=msg.authextra,
+                                         resumed=msg.resumed,
+                                         resumable=msg.resumable,
+                                         resume_token=msg.resume_token)
                 # firing 'join' *before* running onJoin, so that the
                 # idiom where you "do stuff" in onJoin -- possibly
                 # including self.leave() -- works properly. Besides,
@@ -487,8 +515,15 @@ class ApplicationSession(BaseSession):
 
                 def success(arg):
                     # XXX also: handle async
-                    self.fire('leave', self, details)
-                    return arg
+                    d = self.fire('leave', self, details)
+
+                    def return_arg(_):
+                        return arg
+
+                    def _error(e):
+                        return self._swallow_error(e, "While firing 'leave' event")
+                    txaio.add_callbacks(d, return_arg, _error)
+                    return d
 
                 def _error(e):
                     return self._swallow_error(e, "While firing onLeave")
@@ -568,19 +603,22 @@ class ApplicationSession(BaseSession):
                         handler = subscription.handler
                         topic = msg.topic or subscription.topic
 
-                        if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
+                        if msg.enc_algo:
                             # FIXME: behavior in error cases (no keyring, decrypt issues, URI mismatch, ..)
-                            if not self._keyring:
-                                self.log.warn("received encrypted payload, but no keyring active - ignoring encrypted payload!")
+                            if not self._payload_codec:
+                                self.log.warn("received encoded payload with enc_algo={enc_algo}, but no payload codec active - ignoring encoded payload!", enc_algo=msg.enc_algo)
+                                return
                             else:
                                 try:
-                                    encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
-                                    decrypted_topic, msg.args, msg.kwargs = self._keyring.decrypt(False, topic, encrypted_payload)
+                                    encoded_payload = EncodedPayload(msg.payload, msg.enc_algo, msg.enc_serializer, msg.enc_key)
+                                    decoded_topic, msg.args, msg.kwargs = self._payload_codec.decode(False, topic, encoded_payload)
                                 except Exception as e:
-                                    self.log.warn("failed to decrypt application payload: {error}", error=e)
+                                    self.log.warn("failed to decode application payload encoded with enc_algo={enc_algo}: {error}", error=e, enc_algo=msg.enc_algo)
+                                    return
                                 else:
-                                    if topic != decrypted_topic:
-                                        self.log.warn("envelope topic URI does not match encrypted one")
+                                    if topic != decoded_topic:
+                                        self.log.warn("envelope topic URI does not match encoded one")
+                                        return
 
                         invoke_args = (handler.obj,) if handler.obj else tuple()
                         if msg.args:
@@ -674,16 +712,16 @@ class ApplicationSession(BaseSession):
                     proc = call_request.procedure
                     enc_err = None
 
-                    if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
+                    if msg.enc_algo:
 
-                        if not self._keyring:
-                            log_msg = u"received encrypted payload, but no keyring active"
+                        if not self._payload_codec:
+                            log_msg = u"received encoded payload, but no payload codec active"
                             self.log.warn(log_msg)
-                            enc_err = ApplicationError(ApplicationError.ENC_NO_KEYRING_ACTIVE, log_msg)
+                            enc_err = ApplicationError(ApplicationError.ENC_NO_PAYLOAD_CODEC, log_msg)
                         else:
                             try:
-                                encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
-                                decrypted_proc, msg.args, msg.kwargs = self._keyring.decrypt(True, proc, encrypted_payload)
+                                encoded_payload = EncodedPayload(msg.payload, msg.enc_algo, msg.enc_serializer, msg.enc_key)
+                                decrypted_proc, msg.args, msg.kwargs = self._payload_codec.decode(True, proc, encoded_payload)
                             except Exception as e:
                                 self.log.warn(
                                     "failed to decrypt application payload 1: {err}",
@@ -772,15 +810,15 @@ class ApplicationSession(BaseSession):
                         proc = msg.procedure or registration.procedure
                         enc_err = None
 
-                        if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
-                            if not self._keyring:
+                        if msg.enc_algo:
+                            if not self._payload_codec:
                                 log_msg = u"received encrypted INVOCATION payload, but no keyring active"
                                 self.log.warn(log_msg)
-                                enc_err = ApplicationError(ApplicationError.ENC_NO_KEYRING_ACTIVE, log_msg)
+                                enc_err = ApplicationError(ApplicationError.ENC_NO_PAYLOAD_CODEC, log_msg)
                             else:
                                 try:
-                                    encrypted_payload = EncryptedPayload(msg.enc_algo, msg.enc_key, msg.enc_serializer, msg.payload)
-                                    decrypted_proc, msg.args, msg.kwargs = self._keyring.decrypt(False, proc, encrypted_payload)
+                                    encoded_payload = EncodedPayload(msg.payload, msg.enc_algo, msg.enc_serializer, msg.enc_key)
+                                    decrypted_proc, msg.args, msg.kwargs = self._payload_codec.decode(False, proc, encoded_payload)
                                 except Exception as e:
                                     self.log.warn(
                                         "failed to decrypt INVOCATION payload: {err}",
@@ -826,19 +864,19 @@ class ApplicationSession(BaseSession):
                                 if msg.receive_progress:
 
                                     def progress(*args, **kwargs):
-                                        encrypted_payload = None
-                                        if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
-                                            if not self._keyring:
+                                        encoded_payload = None
+                                        if msg.enc_algo:
+                                            if not self._payload_codec:
                                                 raise Exception(u"trying to send encrypted payload, but no keyring active")
-                                            encrypted_payload = self._keyring.encrypt(False, proc, args, kwargs)
+                                            encoded_payload = self._payload_codec.encode(False, proc, args, kwargs)
 
-                                        if encrypted_payload:
+                                        if encoded_payload:
                                             progress_msg = message.Yield(msg.request,
-                                                                         payload=encrypted_payload.payload,
+                                                                         payload=encoded_payload.payload,
                                                                          progress=True,
-                                                                         enc_algo=encrypted_payload.algo,
-                                                                         enc_key=encrypted_payload.pkey,
-                                                                         enc_serializer=encrypted_payload.serializer)
+                                                                         enc_algo=encoded_payload.enc_algo,
+                                                                         enc_key=encoded_payload.enc_key,
+                                                                         enc_serializer=encoded_payload.enc_serializer)
                                         else:
                                             progress_msg = message.Yield(msg.request,
                                                                          args=args,
@@ -856,29 +894,29 @@ class ApplicationSession(BaseSession):
                             def success(res):
                                 del self._invocations[msg.request]
 
-                                encrypted_payload = None
-                                if msg.enc_algo == message.PAYLOAD_ENC_CRYPTO_BOX:
-                                    if not self._keyring:
+                                encoded_payload = None
+                                if msg.enc_algo:
+                                    if not self._payload_codec:
                                         log_msg = u"trying to send encrypted payload, but no keyring active"
                                         self.log.warn(log_msg)
                                     else:
                                         try:
                                             if isinstance(res, types.CallResult):
-                                                encrypted_payload = self._keyring.encrypt(False, proc, res.results, res.kwresults)
+                                                encoded_payload = self._payload_codec.encode(False, proc, res.results, res.kwresults)
                                             else:
-                                                encrypted_payload = self._keyring.encrypt(False, proc, [res])
+                                                encoded_payload = self._payload_codec.encode(False, proc, [res])
                                         except Exception as e:
                                             self.log.warn(
                                                 "failed to encrypt application payload: {err}",
                                                 err=e,
                                             )
 
-                                if encrypted_payload:
+                                if encoded_payload:
                                     reply = message.Yield(msg.request,
-                                                          payload=encrypted_payload.payload,
-                                                          enc_algo=encrypted_payload.algo,
-                                                          enc_key=encrypted_payload.pkey,
-                                                          enc_serializer=encrypted_payload.serializer)
+                                                          payload=encoded_payload.payload,
+                                                          enc_algo=encoded_payload.enc_algo,
+                                                          enc_key=encoded_payload.enc_key,
+                                                          enc_serializer=encoded_payload.enc_serializer)
                                 else:
                                     if isinstance(res, types.CallResult):
                                         reply = message.Yield(msg.request,
@@ -1025,7 +1063,7 @@ class ApplicationSession(BaseSession):
 
                 raise ProtocolError("Unexpected message {0}".format(msg.__class__))
 
-    # noinspection PyUnusedLocal
+    @public
     def onClose(self, wasClean):
         """
         Implements :func:`autobahn.wamp.interfaces.ITransportHandler.onClose`
@@ -1060,31 +1098,77 @@ class ApplicationSession(BaseSession):
 
         def _error(e):
             return self._swallow_error(e, "While firing onDisconnect")
-        txaio.add_callbacks(d, None, _error)
+        txaio.add_callbacks(d, success, _error)
 
+    @public
     def onChallenge(self, challenge):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.onChallenge`
         """
         raise Exception("received authentication challenge, but onChallenge not implemented")
 
+    @public
     def onJoin(self, details):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.onJoin`
         """
 
+    def _errback_outstanding_requests(self, exc):
+        """
+        Errback any still outstanding requests with exc.
+        """
+        d = txaio.create_future_success(None)
+        all_requests = [
+            self._publish_reqs,
+            self._subscribe_reqs,
+            self._unsubscribe_reqs,
+            self._call_reqs,
+            self._register_reqs,
+            self._unregister_reqs
+        ]
+        outstanding = []
+        for requests in all_requests:
+            outstanding.extend(requests.values())
+            requests.clear()
+
+        if outstanding:
+            self.log.info(
+                'Cancelling {count} outstanding requests',
+                count=len(outstanding),
+            )
+        for request in outstanding:
+            self.log.debug(
+                'cleaning up outstanding {request_type} request {request_id}, '
+                'firing errback on user handler {request_on_reply}',
+                request_on_reply=request.on_reply,
+                request_id=request.request_id,
+                request_type=request.__class__.__name__,
+            )
+            txaio.reject(request.on_reply, exc)
+            # wait for any async-ness in the error handlers for on_reply
+            txaio.add_callbacks(d, lambda _: request.on_reply, lambda _: request.on_reply)
+        return d
+
+    @public
     def onLeave(self, details):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.onLeave`
         """
-        if details.reason.startswith('wamp.error.'):
-            self.log.error('{reason}: {wamp_message}', reason=details.reason, wamp_message=details.message)
+        if details.reason != CloseDetails.REASON_DEFAULT:
+            self.log.warn('session closed with reason {reason} [{message}]', reason=details.reason, message=details.message)
 
-        if self._transport:
-            self.disconnect()
-        # do we ever call onLeave with a valid transport?
+        # fire ApplicationError on any currently outstanding requests
+        exc = ApplicationError(details.reason, details.message)
+        d = self._errback_outstanding_requests(exc)
 
-    def leave(self, reason=None, log_message=None):
+        def disconnect(_):
+            if self._transport:
+                self.disconnect()
+        txaio.add_callbacks(d, disconnect, disconnect)
+        return d
+
+    @public
+    def leave(self, reason=None, message=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.leave`
         """
@@ -1094,7 +1178,7 @@ class ApplicationSession(BaseSession):
         if not self._goodbye_sent:
             if not reason:
                 reason = u"wamp.close.normal"
-            msg = wamp.message.Goodbye(reason=reason, message=log_message)
+            msg = wamp.message.Goodbye(reason=reason, message=message)
             self._transport.send(msg)
             self._goodbye_sent = True
             # deferred that fires when transport actually hits CLOSED
@@ -1103,18 +1187,22 @@ class ApplicationSession(BaseSession):
         else:
             raise SessionNotReady(u"session was alread requested to leave")
 
+    @public
     def onDisconnect(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ISession.onDisconnect`
         """
-        pass  # return self.fire('disconnect', self, True)
+        # fire TransportLost on any _still_ outstanding requests
+        # (these should have been already cleaned up in onLeave() - when
+        # this actually has fired)
+        exc = exception.TransportLost()
+        self._errback_outstanding_requests(exc)
 
+    @public
     def publish(self, topic, *args, **kwargs):
         """
         Implements :func:`autobahn.wamp.interfaces.IPublisher.publish`
         """
-        if six.PY2 and type(topic) == str:
-            topic = six.u(topic)
         assert(type(topic) == six.text_type)
 
         if not self._transport:
@@ -1126,26 +1214,26 @@ class ApplicationSession(BaseSession):
 
         request_id = self._request_id_gen.next()
 
-        encrypted_payload = None
-        if self._keyring:
-            encrypted_payload = self._keyring.encrypt(True, topic, args, kwargs)
+        encoded_payload = None
+        if self._payload_codec:
+            encoded_payload = self._payload_codec.encode(True, topic, args, kwargs)
 
-        if encrypted_payload:
+        if encoded_payload:
             if options:
                 msg = message.Publish(request_id,
                                       topic,
-                                      payload=encrypted_payload.payload,
-                                      enc_algo=encrypted_payload.algo,
-                                      enc_key=encrypted_payload.pkey,
-                                      enc_serializer=encrypted_payload.serializer,
+                                      payload=encoded_payload.payload,
+                                      enc_algo=encoded_payload.enc_algo,
+                                      enc_key=encoded_payload.enc_key,
+                                      enc_serializer=encoded_payload.enc_serializer,
                                       **options.message_attr())
             else:
                 msg = message.Publish(request_id,
                                       topic,
-                                      payload=encrypted_payload.payload,
-                                      enc_algo=encrypted_payload.algo,
-                                      enc_key=encrypted_payload.pkey,
-                                      enc_serializer=encrypted_payload.serializer)
+                                      payload=encoded_payload.payload,
+                                      enc_algo=encoded_payload.enc_algo,
+                                      enc_key=encoded_payload.enc_key,
+                                      enc_serializer=encoded_payload.enc_serializer)
         else:
             if options:
                 msg = message.Publish(request_id,
@@ -1162,7 +1250,7 @@ class ApplicationSession(BaseSession):
         if options and options.acknowledge:
             # only acknowledged publications expect a reply ..
             on_reply = txaio.create_future()
-            self._publish_reqs[request_id] = PublishRequest(request_id, on_reply, was_encrypted=(encrypted_payload is not None))
+            self._publish_reqs[request_id] = PublishRequest(request_id, on_reply, was_encrypted=(encoded_payload is not None))
         else:
             on_reply = None
 
@@ -1183,13 +1271,12 @@ class ApplicationSession(BaseSession):
 
         return on_reply
 
+    @public
     def subscribe(self, handler, topic=None, options=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ISubscriber.subscribe`
         """
         assert((callable(handler) and topic is not None) or hasattr(handler, '__class__'))
-        if topic and six.PY2 and type(topic) == str:
-            topic = six.u(topic)
         assert(topic is None or type(topic) == six.text_type)
         assert(options is None or isinstance(options, types.SubscribeOptions))
 
@@ -1264,13 +1351,12 @@ class ApplicationSession(BaseSession):
             # there are still handlers active on the subscription!
             return txaio.create_future_success(scount)
 
+    @public
     def call(self, procedure, *args, **kwargs):
         """
         Implements :func:`autobahn.wamp.interfaces.ICaller.call`
         """
-        if six.PY2 and type(procedure) == str:
-            procedure = six.u(procedure)
-        assert(isinstance(procedure, six.text_type))
+        assert(type(procedure) == six.text_type)
 
         if not self._transport:
             raise exception.TransportLost()
@@ -1281,26 +1367,30 @@ class ApplicationSession(BaseSession):
 
         request_id = self._request_id_gen.next()
 
-        encrypted_payload = None
-        if self._keyring:
-            encrypted_payload = self._keyring.encrypt(True, procedure, args, kwargs)
+        encoded_payload = None
+        if self._payload_codec:
+            try:
+                encoded_payload = self._payload_codec.encode(True, procedure, args, kwargs)
+            except:
+                self.log.failure()
+                raise
 
-        if encrypted_payload:
+        if encoded_payload:
             if options:
                 msg = message.Call(request_id,
                                    procedure,
-                                   payload=encrypted_payload.payload,
-                                   enc_algo=encrypted_payload.algo,
-                                   enc_key=encrypted_payload.pkey,
-                                   enc_serializer=encrypted_payload.serializer,
+                                   payload=encoded_payload.payload,
+                                   enc_algo=encoded_payload.enc_algo,
+                                   enc_key=encoded_payload.enc_key,
+                                   enc_serializer=encoded_payload.enc_serializer,
                                    **options.message_attr())
             else:
                 msg = message.Call(request_id,
                                    procedure,
-                                   payload=encrypted_payload.payload,
-                                   enc_algo=encrypted_payload.algo,
-                                   enc_key=encrypted_payload.pkey,
-                                   enc_serializer=encrypted_payload.serializer)
+                                   payload=encoded_payload.payload,
+                                   enc_algo=encoded_payload.enc_algo,
+                                   enc_key=encoded_payload.enc_key,
+                                   enc_serializer=encoded_payload.enc_serializer)
         else:
             if options:
                 msg = message.Call(request_id,
@@ -1340,13 +1430,12 @@ class ApplicationSession(BaseSession):
 
         return on_reply
 
+    @public
     def register(self, endpoint, procedure=None, options=None):
         """
         Implements :func:`autobahn.wamp.interfaces.ICallee.register`
         """
         assert((callable(endpoint) and procedure is not None) or hasattr(endpoint, '__class__'))
-        if procedure and six.PY2 and type(procedure) == str:
-            procedure = six.u(procedure)
         assert(procedure is None or type(procedure) == six.text_type)
         assert(options is None or isinstance(options, types.RegisterOptions))
 
@@ -1409,8 +1498,8 @@ class ApplicationSession(BaseSession):
         return on_reply
 
 
-# IApplicationSession.register collides with the abc.ABCMeta.register method
-# IApplicationSession.register(ApplicationSession)
+# ISession.register collides with the abc.ABCMeta.register method
+# ISession.register(ApplicationSession)
 
 
 class ApplicationSessionFactory(object):
@@ -1420,8 +1509,8 @@ class ApplicationSessionFactory(object):
 
     session = ApplicationSession
     """
-   WAMP application session class to be used in this factory.
-   """
+    WAMP application session class to be used in this factory.
+    """
 
     def __init__(self, config=None):
         """
